@@ -126,6 +126,11 @@ void error_message(unsigned char *hdr, unsigned char *txt)
     fprintf(stderr, "\nERROR: %-15.15s %s", hdr, txt);
 }
 
+void error_message_value(unsigned char *hdr, unsigned char *txt, long value)
+{
+    fprintf(stderr, "\nERROR: %-15.15s %s %ld (0x%lx)", hdr, txt, value, value);
+}
+
 void debug_message(unsigned char *hdr, unsigned char *txt)
 {
 #ifdef _DEBUG
@@ -133,6 +138,13 @@ void debug_message(unsigned char *hdr, unsigned char *txt)
 #endif
 }
 
+void debug_message_value(unsigned char *hdr, unsigned char *txt, long value)
+{
+#ifdef _DEBUG
+    fprintf(stderr, "\nDEBUG: %-15.15s %s %ld (0x%lx)", hdr, txt, value, value);
+#endif
+}
+
 void debug_buffer(unsigned char *hdr, unsigned char *buf)
 {
 #ifdef _DEBUG
@@ -151,17 +163,13 @@ int Uproar_get_memory()
 
 void Uproar_set_memory(int memory)
 {
-    char dbgstr[256];
-    sprintf(dbgstr, "memory=%d", memory);
-    debug_message("Uproar_set_memory", dbgstr);
+    debug_message_value("Uproar_set_memory", "memory", memory);
     uproar_memory = memory;
 }
 
 void Set_download_bitrate(int bitrate)
 {
-    char dbgstr[256];
-    sprintf(dbgstr, "bitrate=%d", bitrate);
-    debug_message("Set_download_bitrate", dbgstr);
+    debug_message_value("Set_download_bitrate", "bitrate", bitrate);
     download_bitrate = bitrate;
 }
 
@@ -295,13 +303,17 @@ int uproar_control(usb_dev_handle * dh, int request, unsigned char lsb,
 			     timeout);
 
     if (retval < 0) {
-	error_message("uproar_control", "usb_control_msg");
+	error_message("uproar_control", "usb_control_msg failed");
 	error_message("uproar_control", usb_strerror());
 	return -1;
     }
 
     if (data_buffer[0] != 0x40) {
-	error_message("uproar_control", "did not get 0x40");
+	error_message_value(
+	    "uproar_control",
+	    "expected 0x40, got",
+	    data_buffer[0]
+	);
 	if (count == 0) {
 	    goto retry;
 	    count++;
@@ -333,7 +345,7 @@ int uproar_send(usb_dev_handle * dh, int request, unsigned char lsb,
 			    USB_SHORT_TIMEOUT);
 
     if (retval < 0) {
-	error_message("uproar_send", "usb_bulk_write");
+	error_message("uproar_send", "usb_bulk_write failed");
 	error_message("uproar_send", usb_strerror());
     }
 
@@ -364,7 +376,7 @@ int uproar_recv(usb_dev_handle * dh, int request, unsigned char lsb,
 			   USB_SHORT_TIMEOUT);
 
     if (retval < 0) {
-	error_message("uproar_recv", "usb_bulk_read");
+	error_message("uproar_recv", "usb_bulk_read failed");
 	error_message("uproar_recv", usb_strerror());
 	return retval;
     }
@@ -408,12 +420,33 @@ int uproar_open()
     Uproar_handle = usb_open(Uproar_dev);
     if (Uproar_handle == NULL) {
 	error_message("uproar_open", "usb_open failed");
+	error_message("uproar_open", usb_strerror());
 	return -1;
     }
 
     retval = usb_claim_interface(Uproar_handle, 0);
     if (retval < 0) {
 	error_message("uproar_open", "usb_claim_interface failed");
+	error_message("uproar_open", usb_strerror());
+	return -1;
+    }
+
+    /* Clear possible stalls on the endpoints.
+     * Required for at least the combination Chic MP10 and linux kernel 2.6.7.
+     * Not neccessary with 2.4.x, but a good idea anyway in case the device
+     * got messed up somehow.
+     */
+    retval = usb_clear_halt(Uproar_handle, read_endpoint);
+    if (retval < 0) {
+	error_message("uproar_open", "usb_clear_halt on read_endpoint failed");
+	error_message("uproar_open", usb_strerror());
+	return -1;
+    }
+
+    retval = usb_clear_halt(Uproar_handle, write_endpoint);
+    if (retval < 0) {
+	error_message("uproar_open", "usb_clear_halt on write endpoint failed");
+	error_message("uproar_open", usb_strerror());
 	return -1;
     }
 
@@ -882,7 +915,7 @@ int Uproar_delete_track(int tracknum)
     if (!reply_as_expected) {
 	error_message("Uproar_delete_track", "unexpected recv buffer 2");
 	sprintf(send_buffer, "buffer 4 and 5, %x %x", recv_buffer[4], recv_buffer[5]);
-	error_message("Actual posponse", send_buffer);
+	error_message("Actual response", send_buffer);
 	uproar_close();
 	return UPROAR_ERROR;
     }
@@ -983,7 +1016,7 @@ int Uproar_reformat_memory()
 	error_message("Uproar_reformat_memory",
 		      "unexpected recv buffer 2");
 	sprintf(send_buffer, "buffer 4 and 5, %x %x", recv_buffer[4], recv_buffer[5]);
-	error_message("Actual posponse", send_buffer);
+	error_message("Actual response", send_buffer);
 	uproar_close();
 	return UPROAR_ERROR;
     }
@@ -1210,6 +1243,7 @@ int Uproar_download(unsigned char *filepath)
 			   send_track_data_len, USB_LONG_TIMEOUT);
 	if (retval < 0) {
 	    error_message("Uproar_download", "usb_bulk_write failed");
+	    error_message("Uproar_download", usb_strerror());
 	    fclose(fp);
             if (download_bitrate > 0) {
                 unlink(temp_filename);
